/******************************************************************************
 * $Id: TRInfoWindow.cpp 2549 2007-07-30 13:59:23Z charles $
 *
 * Authors:
 *		charles
 *		Fredrik Modéen <fredrik@modeen.se>
 *
 * Copyright (c) 2005-2007 Transmission authors and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *****************************************************************************/

#include "TRInfoWindow.h"
#include "TRApplication.h"

#include <Rect.h>
#include <TranslationUtils.h>

#include <malloc.h>
#include <stdio.h>

TRInfoWindow::TRInfoWindow(const tr_stat_t *status, const tr_info_t *info, const tr_torrent_t *torrentRef, const char *folder) 
	: BWindow(BRect(0, 0, 600, 500), "Torrent Inspector", B_TITLED_WINDOW, B_ASYNCHRONOUS_CONTROLS 
		| B_NOT_ZOOMABLE | B_NOT_RESIZABLE, B_CURRENT_WORKSPACE)
{
	BRect tabRect, /*tabViewRect,*/ mainRect, mainStringRect, boxRect, textRectTab1, /*textRectTab2,*/ textRectTab3, textRectTab4;
	mainRect = Bounds();
	
	font_height fh;
	be_plain_font->GetHeight(&fh);
	
	_trprefview = new BView(mainRect, "TRPrefView", B_FOLLOW_H_CENTER, NULL);
	_trprefview->SetViewColor(216,216,216,0);
	
	// Header string
	//--------------------------------------------------------------
	BFont headerViewFont(be_bold_font);
	headerViewFont.SetSize(18.0f);
	font_height fhView;
	headerViewFont.GetHeight(&fhView);
	mainStringRect.Set(5, 5, mainRect.Width() - 10, fhView.leading + fhView.ascent + 10);

	BStringView *strInfoView = new BStringView(mainStringRect, "header", info->name, B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP);
	strInfoView->SetFont(&headerViewFont);
	strInfoView->SetAlignment(B_ALIGN_CENTER);

	// TabView
	//--------------------------------------------------------------	
	tabRect = mainRect;
	tabRect.bottom = tabRect.bottom - 20;
	tabRect.InsetBy(5,mainStringRect.bottom +5);
	_tabView = new BTabView(tabRect, "tab_view");
	_tabView->SetViewColor(216,216,216,0);       	
	tabRect.top = tabRect.top - mainStringRect.bottom;

	boxRect = BRect(tabRect.left, tabRect.top, tabRect.right-25, tabRect.bottom-75);
	
	//Tab Info
	//--------------------------------------------------------------
	_Info = new BView(tabRect, "Info", B_FOLLOW_H_CENTER, NULL);
	_Info->SetViewColor(216,216,216,0);
	_Info->AddChild(_boxInfo = new BBox(boxRect, NULL, B_FOLLOW_ALL));
	_tabView->AddTab(_Info, new BTab());
	
	//Tab Activity
	//--------------------------------------------------------------
	_activity = new BView(tabRect, "Activity", B_FOLLOW_H_CENTER, NULL);
	_activity->SetViewColor(216,216,216,0);
	_activity->AddChild(_boxActivity = new BBox(boxRect, NULL, B_FOLLOW_ALL));
	_tabView->AddTab(_activity, new BTab());
	
	//Tab Peers
	//--------------------------------------------------------------
	_peers = new BView(tabRect, "Peers", B_FOLLOW_H_CENTER, NULL);
	_peers->SetViewColor(216,216,216,0);
	_peers->AddChild(_boxPeers = new BBox(boxRect, NULL, B_FOLLOW_ALL));
	_tabView->AddTab(_peers, new BTab());
	
	//Tab Files
	//--------------------------------------------------------------	
	_files = new BView(tabRect, "Files", B_FOLLOW_H_CENTER, NULL);
	_files->SetViewColor(216,216,216,0);
	_files->AddChild(_boxFiles = new BBox(boxRect, NULL, B_FOLLOW_ALL));
	_tabView->AddTab(_files, new BTab());

	//Tab Options
	//--------------------------------------------------------------	
	_options = new BView(tabRect, "Options", B_FOLLOW_H_CENTER, NULL);
	_options->SetViewColor(216,216,216,0);
	_options->AddChild(_boxOptions = new BBox(boxRect, NULL, B_FOLLOW_ALL));
	_tabView->AddTab(_options, new BTab());
	
	//--------------------------------------------------------------
	//stuff that are put on the info Tab
	//--------------------------------------------------------------
	BFont headerFont(be_bold_font);
	headerFont.SetSize(14.0f);
	font_height fh1;
	headerFont.GetHeight(&fh1);
	textRectTab1.Set(5, 5, boxRect.Width() - 10, fh1.leading + fh1.ascent + 10);
		
	BString strTracker(status->tracker->address);
	strTracker << ":" << status->tracker->port << status->tracker->announce;

	BString strPieceSize("");
	StringForFileSize(info->pieceSize, &strPieceSize);

	BString strTotalSize("");
	StringForFileSize(info->totalSize, &strTotalSize);

	BString strDownloaded("");
	StringForFileSize(status->downloaded, &strDownloaded);

	BString strUploaded("");
	StringForFileSize(status->uploaded, &strUploaded);
	BString infoStr("");
	infoStr << "Tracker: " << strTracker << "\n"
	        << "Announce: " << status->tracker->announce << "\n"
//	        << "Piece Size: " << strPieceSize << "\n"
	        << "Pieces: " << info->pieceCount <<  ", " << strPieceSize << "\n"
	        << "Total Size: " << strTotalSize << "\n"
	        << "\n"
	        << "Folder: " << folder << "\n"
	        << "Downloaded: " << strDownloaded << "\n"
	        << "Uploaded: " << strUploaded << "\n";
//	        << "Creator: " << info->creator << "\n";

//	infoStr = *new BString("");
//	infoStr << info->name << " (" << info->totalSize << ")";
	BStringView *strView = new BStringView(textRectTab1, "header", info->name, B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP);
	strView->SetFont(&headerFont);
	strView->SetAlignment(B_ALIGN_CENTER);

	textRectTab1.top = fh1.ascent + fh1.descent+10;
	textRectTab1.bottom = boxRect.bottom-10;
	BRect textRect = textRectTab1;
	textRect.top = 0;
	
	BTextView *txtView = new BTextView(textRectTab1, "infoText", textRect, B_FOLLOW_LEFT | B_FOLLOW_TOP);
	txtView->MakeEditable(false);
	txtView->SetText(infoStr.String());
	txtView->SetViewColor(216,216,216,0);

	Lock();
	_boxInfo->AddChild(strView);
	_boxInfo->AddChild(txtView);
	Unlock();
	
	//stuff that are put on the Activity Tab
	//--------------------------------------------------------------

	//stuff that are put on the Peers Tab
	//--------------------------------------------------------------
	textRectTab3.Set(5, 5, boxRect.Width() - 10, fh.leading + fh.ascent + 10);

	// create the list view 
	fColumnListTab2 = new BColumnListView (Bounds(), "ColumnList", B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM,
		B_WILL_DRAW |B_NAVIGABLE);

	// add columns 
	//--------
	fColumnListTab2->AddColumn(new BStringColumn("IP Adress", 300, 20, 500, 0),0);
	
	//-------
	fColumnListTab2->AddColumn(new TRColumn("Client", 50, 20, 100),1);

	//------------------
	fColumnListTab2->AddColumn(new TRColumn("%", 50, 20, 100),2);
	
	//---------

//	CreatePeersList(info);
//	Lock();
	//_boxPeers->AddChild(fColumnList);
//	Unlock();

	//stuff that are put on the Files Tab
	//--------------------------------------------------------------
	textRectTab4.Set(5, 5, boxRect.Width() - 10, fh.leading + fh.ascent + 10);
	Lock();	

	// create the list view 
	fColumnList = new BColumnListView (Bounds(), "ColumnList", B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM,
		B_WILL_DRAW |B_NAVIGABLE);

	// add columns 
//--------
	fColumnList->AddColumn(new BStringColumn("Name", 300, 20, 500, 0),0);
	
//-------
	fTRColumView1 = new TRColumn("DL", 50, 20, 100);
	BBitmap *bitmap = BTranslationUtils::GetBitmap('RAWT',"BEOS:GO");
	fTRColumView1->AddBitmapState(0, bitmap);
	bitmap = NULL;
	
	bitmap = BTranslationUtils::GetBitmap('RAWT',"BEOS:STOP");
	if(bitmap != NULL)
		fTRColumView1->AddBitmapState(1, bitmap);
	bitmap = NULL;

	fColumnList->AddColumn(fTRColumView1,1);

//------------------
	fTRColumView2 = new TRColumn("Rank", 50, 20, 100);

	bitmap = BTranslationUtils::GetBitmap('RAWT',"BEOS:PRIORITYNORMAL");
	if(bitmap != NULL)
		fTRColumView2->AddBitmapState(0, bitmap);
	bitmap = NULL;
	
	bitmap = BTranslationUtils::GetBitmap('RAWT',"BEOS:PRIORITYHIGH");
	if(bitmap != NULL)
		fTRColumView2->AddBitmapState(1, bitmap);
	bitmap = NULL;
	
	bitmap = BTranslationUtils::GetBitmap('RAWT',"BEOS:PRIORITYLOW");
	if(bitmap != NULL)
		fTRColumView2->AddBitmapState(2, bitmap);
		
	fColumnList->AddColumn(fTRColumView2,2);
//---------

	CreateFileList(info);
	_boxFiles->AddChild(fColumnList);
	Unlock();
	
	//stuff that are put on the Options Tab
	//--------------------------------------------------------------

	//--------------------------------------------------------------
	Lock();
	_trprefview->AddChild(strInfoView);
	_trprefview->AddChild(_tabView);
	
	// Buttons
	//--------------------------------------------------------------
	BRect buttonRect = BRect(tabRect.left, tabRect.bottom+10, tabRect.left+60, tabRect.bottom+50);
	_trprefview->AddChild(new BButton(buttonRect, "btnDefault", "Defaults", new BMessage(TR_PREF_DEFAULTS)));

	buttonRect.OffsetBy(buttonRect.Width() + 10, 0);
	_trprefview->AddChild(new BButton(buttonRect, "btnCancel", "Cancel", new BMessage(TR_PREF_CANCEL)));
	
	buttonRect.OffsetBy(buttonRect.Width() + 10, 0);	
	BButton *btnSave = new BButton(buttonRect, "btnSave", "Apply", new BMessage(TR_PREF_SAVE));
	btnSave->MakeDefault(true);
	_trprefview->AddChild(btnSave);
	AddChild(_trprefview);
	Unlock();
}

TRInfoWindow::~TRInfoWindow() {}

void TRInfoWindow::FrameResized(float, float) {}


void TRInfoWindow::StringForFileSize(uint64_t size, BString *str) {
	char *s = (char*)calloc(512, sizeof(char));
	if (size < 1024) {
		sprintf(s, "%lld bytes", size);
	} else if (size < 1048576) {
		sprintf(s, "%lld.%lld KB", size / 1024, (size % 1024 ) / 103);
	} else if (size < 1073741824 ) {
		sprintf(s, "%lld.%lld MB", size / 1048576, (size % 1048576) / 104858);
	} else {
		sprintf(s, "%lld.%lld GB", size / 1073741824, (size % 1073741824) / 107374183);
	}
	
	str->SetTo(s);
	free(s);
}

void TRInfoWindow::CreateFileList(const tr_info_t *info)
{
	BRow *cRow;
	tr_file_t file;
	int count, i;
	BString str;

	count = info->fileCount;
    
    for (i = 0; i < count; i++)
    {
    	file = info->files[i];
		str = *new BString(file.name);
      	if (info->multifile)
        {
        	str = str.RemoveFirst(info->name);
	       	str = str.RemoveFirst("/");
        }

		cRow = new BRow();
		cRow->SetField(new BStringField(str.String()), 0);
		cRow->SetField(new TRField(fTRColumView1->GetStartBitmap()), 1);
		cRow->SetField(new TRField(fTRColumView2->GetStartBitmap()), 2);
		fColumnList->AddRow(cRow, (int32)i);
    }
}

void TRInfoWindow::CreatePeersList(const tr_torrent_t *torrentRef)
{
/*	int totalPeers, i;
    tr_peer_stat_t * peers = tr_torrentPeers(fHandle, &totalPeers);
    
    NSMutableArray * peerDics = [NSMutableArray arrayWithCapacity: totalPeers];
    NSMutableDictionary * dic;
    
    tr_peer_stat_t * peer;
    for (i = 0; i < totalPeers; i++)
    {
        peer = &peers[i];
        
        dic = [NSMutableDictionary dictionaryWithObjectsAndKeys:
            [NSNumber numberWithBool: peer->isConnected], @"Connected",
            [NSNumber numberWithInt: peer->from], @"From",
            [NSString stringWithCString: (char *) peer->addr encoding: NSUTF8StringEncoding], @"IP",
            [NSNumber numberWithInt: peer->port], @"Port", nil];
        
        if (peer->isConnected)
        {
            [dic setObject: [NSNumber numberWithFloat: peer->progress] forKey: @"Progress"];
            
            if (peer->isDownloading)
                [dic setObject: [NSNumber numberWithFloat: peer->uploadToRate] forKey: @"UL To Rate"];
            if (peer->isUploading)
                [dic setObject: [NSNumber numberWithFloat: peer->downloadFromRate] forKey: @"DL From Rate"];
            
            [dic setObject: [NSString stringWithCString: (char *) peer->client encoding: NSUTF8StringEncoding] forKey: @"Client"];
        }
        else
            [dic setObject: @"" forKey: @"Client"]; //needed to be set here for client sort
        
        [peerDics addObject: dic];
    }
    
    tr_torrentPeersFree(peers, totalPeers);
    
    return peerDics;*/
//	BRow *cRow;
//	tr_file_t file;
//	int count, i;
//	BString str;

//	int totalPeers, i;
//    tr_peer_stat_t * peers = tr_torrentPeers(fHandle, &totalPeers);
/*	count = info->fileCount;
    
    for (i = 0; i < count; i++)
    {
    	file = info->files[i];
		str = *new BString(file.name);
      	if (info->multifile)
        {
        	str = str.RemoveFirst(info->name);
	       	str = str.RemoveFirst("/");
        }

		cRow = new BRow();
		cRow->SetField(new BStringField(str.String()), 0);
		cRow->SetField(new TRField(fTRColumView1->GetStartBitmap()), 1);
		cRow->SetField(new TRField(fTRColumView2->GetStartBitmap()), 2);
		fColumnList->AddRow(cRow, (int32)i);
    }*/
}

void TRInfoWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what){
		case TR_PREF_SAVE:{
			WritePrefs();
		}break;
		case TR_PREF_CANCEL:{
			Hide();
		}break;
		case TR_PREF_DEFAULTS:{
			SetDefaults();
		}break;
		default: {
			BWindow::MessageReceived(msg);
		}break;
	}
}

void TRInfoWindow::SetDefaults()
{
	TRField *bitmapField;
	BField* fieldDL;
	BField* fieldPrio;
	BRow * row;
	
	uint32 count = fColumnList->CountRows();
	for(uint32 i = 0; i < count;i++)
	{	
		row = fColumnList->RowAt(i);
		
		fieldDL = row->GetField(1);
		bitmapField = static_cast<TRField *>(fieldDL);
		bitmapField->SetBitmap(fTRColumView1->GetBitmapFromState(0));
		
		fieldPrio = row->GetField(2);
		bitmapField = static_cast<TRField *>(fieldPrio);
		bitmapField->SetBitmap(fTRColumView2->GetBitmapFromState(0));
	}
	//Must reread all fields some how.
}

void TRInfoWindow::ReadPrefs() 
{
/*	if (Lock()){
		Prefs prefs(TRANSMISSION_SETTINGS);
		BString str("");
		if (prefs.FindString("download.folder", &str) != B_OK){
			str << DOWNLOADFOLDER;
			prefs.SetString("download.folder", str.String());
		}
		_txtFolder->SetText(str.String());

		//-----------------------------------------------------------
		int32 port;
		if (prefs.FindInt32("transmission.bindPort", &port) != B_OK){
			port = atoi(PORT);
			prefs.SetInt32("transmission.bindPort", port);
		}
		
		str = "";
		str << port;
		_txtPort->SetText(str.String());

		//---------------------------------------------------------------------
		int32 upload;
		if (prefs.FindInt32("transmission.totalUploadLimit", &upload) != B_OK){
			upload = atoi(TOTALUPLOADLIMIT);
			prefs.SetInt32("transmission.totalUploadLimit", upload);			
		}
		str = "";
		str << upload;
		_txtTotalUpload->SetText(str.String());
		
		//-------------------------------------------------------------------------
		int32 download;
		if (prefs.FindInt32("transmission.totalDownloadLimit", &download) != B_OK){
			download = atoi(TOTALDOWNLOADLIMIT);
			prefs.SetInt32("transmission.totalDownloadLimit", download);			
		}
		str = "";
		str << download;
		_txtTotalDownload->SetText(str.String());*/
/*
		//-----------------------------------------------------------------------		
		bool checkBoxDownload;
		if (prefs.FindBool("transmission.checkBoxDownloadLimit", &checkBoxDownload) != B_OK){
			checkBoxDownload = CHECKBOXUNLIMITDOWNLOAD;
			prefs.SetBool("transmission.checkBoxDownloadLimit", checkBoxDownload);			
		}
		
		_unlimitDownload->SetValue(checkBoxDownload);
		
		if(checkBoxDownload)
			_txtTotalDownload->SetEnabled(!checkBoxDownload);

		//-----------------------------------------------------------------------		
		bool checkBoxUpload;
		if (prefs.FindBool("transmission.checkBoxUploadLimit", &checkBoxUpload) != B_OK){
			checkBoxUpload = CHECKBOXUNLIMITUPLOAD;
			prefs.SetBool("transmission.checkBoxUploadLimit", checkBoxUpload);			
		}
		
		_unlimitUpload->SetValue(checkBoxUpload);
		
		if(checkBoxUpload)
			_txtTotalUpload->SetEnabled(!checkBoxUpload);
*/		
//		Unlock();
//	}
}


bool TRInfoWindow::WritePrefs()
{
	bool valid = true;
	
/*	int port = atoi(_txtPort->Text());
	if (port <= 0){
		valid = false;
		_txtPort->MakeFocus(true);
	}

	const char* uploadStr = _txtTotalUpload->Text();
	int totaluploadLimit = atoi(_txtTotalUpload->Text());
	int totaldownloadLimit = atoi(_txtTotalDownload->Text());

	for (uint i = 0; i < strlen(uploadStr) && valid; i++){
		if (!(uploadStr[i] >= '0' && uploadStr[i] <= '9')){
			valid = false;
			_txtTotalUpload->MakeFocus(true);
		}
	}
	
	if (valid){
		Prefs prefs(TRANSMISSION_SETTINGS);
		
		prefs.SetInt32("transmission.bindPort", (int32)port);
		prefs.SetString("download.folder", _txtFolder->Text());
		prefs.SetInt32("transmission.totaluploadLimit", (int32)totaluploadLimit);
		prefs.SetInt32("transmission.totalDownloadLimit", (int32)totaldownloadLimit);			
//		prefs.SetBool("transmission.checkBoxDownloadLimit", _unlimitDownload->Value());
//		prefs.SetBool("transmission.checkBoxUploadLimit", _unlimitUpload->Value());
		
		be_app_messenger.SendMessage(new BMessage(TR_RELOAD_SETTINGS));
	}*/
	
	return valid;
}
