
/******************************************************************************
 * $Id: TRTorrPrefWindow.cpp
 *
 * Authors:
 *		?
 *		Fredrik Modéen <fredrik@modeen.se>
 *
 * Copyright (c) 2005-2007 Transmission authors and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *****************************************************************************/

#include "TRPrefsWindow.h"

TRPrefsWindow::TRPrefsWindow() 
	:BWindow(BRect(0, 0, 450, 300), "Settings", B_TITLED_WINDOW, B_ASYNCHRONOUS_CONTROLS 
		| B_NOT_ZOOMABLE | B_NOT_RESIZABLE, B_CURRENT_WORKSPACE)
{
	BRect tabRect, mainRect, buttonRect, boxRect, textRectTab1, textRectTab2, textRectTab3, textRectTab4;
	BTab *tab;

	mainRect = Bounds();
	
	font_height fh;
	be_plain_font->GetHeight(&fh);
	
	_trprefview = new BView(mainRect, "TRPrefView", B_FOLLOW_H_CENTER, NULL);
	_trprefview->SetViewColor(216,216,216,0);
	
	tabRect.Set(mainRect.left, mainRect.top, mainRect.right, mainRect.bottom-50);
	tabRect.InsetBy(5,5);
	
	_tabView = new BTabView(tabRect, "tab_view");
	_tabView->SetViewColor(216,216,216,0);          
	
	// Buttons
	//--------------------------------------------------------------
	buttonRect = BRect(tabRect.left, tabRect.bottom+10, tabRect.left+60, tabRect.bottom+50);
	_btnDefaults = new BButton(buttonRect, "btnDefault", "Defaults", new BMessage(TR_PREF_DEFAULTS));
	_trprefview->AddChild(_btnDefaults);

	buttonRect.OffsetBy(buttonRect.Width() + 10, 0);
	_btnCancel = new BButton(buttonRect, "btnCancel", "Cancel", new BMessage(TR_PREF_CANCEL));
	_trprefview->AddChild(_btnCancel);
	
	buttonRect.OffsetBy(buttonRect.Width() + 10, 0);	
	_btnSave = new BButton(buttonRect, "btnSave", "Apply", new BMessage(TR_PREF_SAVE));
	_btnSave->MakeDefault(true);
	_trprefview->AddChild(_btnSave);
	
	boxRect = BRect(tabRect.left, tabRect.top, tabRect.right-25, tabRect.bottom-45);

	//Tab General
	//--------------------------------------------------------------
	tab = new BTab();
	tab->SetLabel("General");
	_general = new BView(tabRect, "General", B_FOLLOW_H_CENTER, NULL);
	_general->SetViewColor(216,216,216,0);
	_boxGeneral = new BBox(boxRect, NULL, B_FOLLOW_ALL);
	_general->AddChild(_boxGeneral);
	_tabView->AddTab(_general, tab);
	
	//Tab Transfers
	//--------------------------------------------------------------
/*	tab = new BTab();
	tab->SetLabel("Transfers");
	_transfer = new BView(tabRect, "Transfers", B_FOLLOW_H_CENTER, NULL);
	_transfer->SetViewColor(216,216,216,0);
	_boxTransfer = new BBox(boxRect, NULL, B_FOLLOW_ALL);
	_transfer->AddChild(_boxTransfer);
	_tabView->AddTab(_transfer, tab);
*/	
	//Tab Bandwidth
	//--------------------------------------------------------------
	tab = new BTab();
	tab->SetLabel("Bandwidth");
	_bandwidth = new BView(tabRect, "Bandwidth", B_FOLLOW_H_CENTER, NULL);
	_bandwidth->SetViewColor(216,216,216,0);
	_boxBandwidth = new BBox(boxRect, NULL, B_FOLLOW_ALL);
	_bandwidth->AddChild(_boxBandwidth);
	_tabView->AddTab(_bandwidth, tab);
	
	//Tab Network
	//--------------------------------------------------------------	
	tab = new BTab();
	tab->SetLabel("Network");
	_network = new BView(tabRect, "Network", B_FOLLOW_H_CENTER, NULL);
	_network->SetViewColor(216,216,216,0);
	_boxNetwork = new BBox(boxRect, NULL, B_FOLLOW_ALL);
	_network->AddChild(_boxNetwork);
	_tabView->AddTab(_network, tab);
	
	//stuff that are put on the General Tab
	//--------------------------------------------------------------
	textRectTab1.Set(5, 5, boxRect.Width() - 10, fh.leading + fh.ascent + 10);
	_txtFolder = new BTextControl(textRectTab1, "txtFolder", "Download directory:", "",
		NULL, B_FOLLOW_LEFT | B_FOLLOW_TOP | B_FOLLOW_RIGHT);
	_boxGeneral->AddChild(_txtFolder);


	//stuff that are put on the Bandwidth Tab
	//--------------------------------------------------------------
	textRectTab3.Set(5, 5, boxRect.Width() - 10, fh.leading + fh.ascent + 10);
	_txtTotalDownload = new BTextControl(textRectTab3, "txtTotalDownload", "Total Download Limit (KB/sec):", "",
		NULL, B_FOLLOW_LEFT | B_FOLLOW_TOP | B_FOLLOW_RIGHT);
	_boxBandwidth->AddChild(_txtTotalDownload);
		
	textRectTab3.Set(textRectTab3.left, textRectTab3.bottom + 10, textRectTab3.right, textRectTab3.bottom + fh.leading + fh.ascent + 15);
	_txtTotalUpload = new BTextControl(textRectTab3, "txtTotalUpload", "Total Upload Limit (KB/sec):", "",
		NULL, B_FOLLOW_LEFT | B_FOLLOW_TOP | B_FOLLOW_RIGHT);
	_boxBandwidth->AddChild(_txtTotalUpload);
	
	//stuff that are put on the Network Tab
	//--------------------------------------------------------------
	textRectTab4.Set(5, 5, boxRect.Width() - 10, fh.leading + fh.ascent + 10);
	_txtPort = new BTextControl(textRectTab4, "txtPort", "Listen On Port:", "", 
		NULL, B_FOLLOW_LEFT | B_FOLLOW_TOP | B_FOLLOW_RIGHT);
	_boxNetwork->AddChild(_txtPort);
		
	Lock();
	_trprefview->AddChild(_tabView);
	AddChild(_trprefview);
	Unlock();
}


TRPrefsWindow::~TRPrefsWindow()
{
}


void TRPrefsWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what){
		case TR_PREF_SAVE:{
			if (WritePrefs()){
				Hide();
			}else{
				beep();
			}
		}break;
		case TR_PREF_CANCEL:{
			Hide();
		}break;
		case TR_PREF_DEFAULTS:{
			SetDefaults();
		}break; 
/*		case TR_PREF_UNLIMITDOWNLOAD:{
			_txtTotalDownload->SetEnabled(!_unlimitDownload->Value());
		}break;

		case TR_PREF_UNLIMITUPLOAD:{
			_txtTotalUpload->SetEnabled(!_unlimitUpload->Value());
		}break;*/
		default: {
			BWindow::MessageReceived(msg);
		}break;
	}
}


void TRPrefsWindow::Show()
{
	ReadPrefs();
	if (Lock())
	{
		_txtFolder->MakeFocus(true);
		Unlock();
	}
	BWindow::Show();
}

void TRPrefsWindow::SetDefaults()
{
	_txtFolder->SetText(DOWNLOADFOLDER);
	_txtPort->SetText(PORT);
	_txtTotalUpload->SetText(TOTALUPLOADLIMIT);
	_txtTotalDownload->SetText(TOTALDOWNLOADLIMIT);
	_txtTotalDownload->SetEnabled(!CHECKBOXUNLIMITDOWNLOAD);
	_txtTotalUpload->SetEnabled(!CHECKBOXUNLIMITUPLOAD);
//	_unlimitDownload->SetValue(CHECKBOXUNLIMITDOWNLOAD);
//	_unlimitUpload->SetValue(CHECKBOXUNLIMITUPLOAD);
}


void TRPrefsWindow::ReadPrefs() 
{
	if (Lock()){
		Prefs prefs(TRANSMISSION_SETTINGS);
		BString str("");
		if (prefs.FindString("download.folder", &str) != B_OK){
			str << DOWNLOADFOLDER;
			prefs.SetString("download.folder", str.String());
		}
		_txtFolder->SetText(str.String());

		//-----------------------------------------------------------
		int32 port;
		if (prefs.FindInt32("transmission.bindPort", &port) != B_OK){
			port = atoi(PORT);
			prefs.SetInt32("transmission.bindPort", port);
		}
		
		str = "";
		str << port;
		_txtPort->SetText(str.String());

		//---------------------------------------------------------------------
		int32 upload;
		if (prefs.FindInt32("transmission.totalUploadLimit", &upload) != B_OK){
			upload = atoi(TOTALUPLOADLIMIT);
			prefs.SetInt32("transmission.totalUploadLimit", upload);			
		}
		str = "";
		str << upload;
		_txtTotalUpload->SetText(str.String());
		
		//-------------------------------------------------------------------------
		int32 download;
		if (prefs.FindInt32("transmission.totalDownloadLimit", &download) != B_OK){
			download = atoi(TOTALDOWNLOADLIMIT);
			prefs.SetInt32("transmission.totalDownloadLimit", download);			
		}
		str = "";
		str << download;
		_txtTotalDownload->SetText(str.String());
/*
		//-----------------------------------------------------------------------		
		bool checkBoxDownload;
		if (prefs.FindBool("transmission.checkBoxDownloadLimit", &checkBoxDownload) != B_OK){
			checkBoxDownload = CHECKBOXUNLIMITDOWNLOAD;
			prefs.SetBool("transmission.checkBoxDownloadLimit", checkBoxDownload);			
		}
		
		_unlimitDownload->SetValue(checkBoxDownload);
		
		if(checkBoxDownload)
			_txtTotalDownload->SetEnabled(!checkBoxDownload);

		//-----------------------------------------------------------------------		
		bool checkBoxUpload;
		if (prefs.FindBool("transmission.checkBoxUploadLimit", &checkBoxUpload) != B_OK){
			checkBoxUpload = CHECKBOXUNLIMITUPLOAD;
			prefs.SetBool("transmission.checkBoxUploadLimit", checkBoxUpload);			
		}
		
		_unlimitUpload->SetValue(checkBoxUpload);
		
		if(checkBoxUpload)
			_txtTotalUpload->SetEnabled(!checkBoxUpload);
*/		
		Unlock();
	}
}


bool TRPrefsWindow::WritePrefs()
{
	bool valid = true;
	
	int port = atoi(_txtPort->Text());
	if (port <= 0){
		valid = false;
		_txtPort->MakeFocus(true);
	}

	const char* uploadStr = _txtTotalUpload->Text();
	int totaluploadLimit = atoi(_txtTotalUpload->Text());
	int totaldownloadLimit = atoi(_txtTotalDownload->Text());

	for (uint i = 0; i < strlen(uploadStr) && valid; i++){
		if (!(uploadStr[i] >= '0' && uploadStr[i] <= '9')){
			valid = false;
			_txtTotalUpload->MakeFocus(true);
		}
	}
	
	if (valid){
		Prefs prefs(TRANSMISSION_SETTINGS);
		
		prefs.SetInt32("transmission.bindPort", (int32)port);
		prefs.SetString("download.folder", _txtFolder->Text());
		prefs.SetInt32("transmission.totaluploadLimit", (int32)totaluploadLimit);
		prefs.SetInt32("transmission.totalDownloadLimit", (int32)totaldownloadLimit);			
//		prefs.SetBool("transmission.checkBoxDownloadLimit", _unlimitDownload->Value());
//		prefs.SetBool("transmission.checkBoxUploadLimit", _unlimitUpload->Value());
		
		be_app_messenger.SendMessage(new BMessage(TR_RELOAD_SETTINGS));
	}
	
	return valid;
}
